#!/bin/bash

# --- Vars ---
SCREEN="/dev/tty0"
CANDEV=can0

# --- End Vars ---

# --- Helpers ---
clearscreen () {
  tput -T xterm clear > $SCREEN
}
setpos () {
   tput -T xterm cup $1 $2 > $SCREEN
}

writetext () {
  echo "$1" > $SCREEN
}

writetext2pos () {
  setpos $1 $2
  #tput -T xterm el > $SCREEN
  writetext "                                                               "
  setpos $1 $2
  writetext "$3"
}

qrundkm () {

  local __res=$1
  local res=""
  local __dk=$2
  
  DKBASEDIR=$(ls -d /media/*/ | head -1)

	
	setpos 28 0
	tput -T xterm-256color el > $SCREEN

	setpos 28 0
	writetext "     Copy to MD900    "
	setpos 28 26
	writetext "   Delete from MD900  "
	#setpos 28 52
	#writetext "      Run Backup      "
	setpos 28 78
	writetext "         Exit         "

	# Read Drumkits on USB
	declare -a DKUSBDIRS
	i=0
	for d in $DKBASEDIR/md900-drumkits/*/
	do
			DKUSBDIRS[i++]="$(basename $d)"
	done

	IFS=$'\n' DKUSBDIRS=($(sort <<<"${DKUSBDIRS[*]}")); unset IFS

	DKUSBCOUNT="${#DKUSBDIRS[@]}"
	let $((DKUSBCOUNT--))

	setpos 26 2
	writetext "From USB: "

	dkusbx=0
	dkusbi=0

	writetext2pos 27 2 "${DKUSBDIRS[$dkusbi]}"

	# Read Drumkits on USB
	declare -a DKMD900DIRS
	i=0
	for d in /var/lib/emi/working/wave/drumkit/*/
	do
			DKMD900DIRS[i++]="$(basename $d)"
	done

	IFS=$'\n' DKMD900DIRS=($(sort <<<"${DKMD900DIRS[*]}")); unset IFS

	DKMD900COUNT="${#DKMD900DIRS[@]}"
	let $((DKMD900COUNT--))

	setpos 26 28
	writetext "From MD900: "

	dkmd900bx=0
	dkmd900i=0

	writetext2pos 27 28 "${DKMD900DIRS[$dkmd900i]}"

	# Menu
	while read line
	do
		# Menu Item Copy from USB
		if [[ $line =~ .+90.+83 ]]; then
			if [ $dkusbi -gt 0 ]; then
				let "dkusbx-=1"
				dkusbi=$(($dkusbx/3))
				setpos 27 2
				writetext "                    "
				setpos 27 2
				writetext "${DKUSBDIRS[$dkusbi]}"
			fi
		fi
		if [[ $line =~ .+90.+82 ]]; then
			if [ $dkusbi -lt $DKUSBCOUNT ]; then
				let "dkusbx+=1"
				dkusbi=$(($dkusbx/3))
				setpos 27 2
				writetext "                    "
				setpos 27 2
				writetext "${DKUSBDIRS[$dkusbi]}"
			fi
		fi
		if [[ $line =~ .+90.+84 ]]; then
			res="CopyToMD900"
			eval $__dk="'${DKUSBDIRS[$dkusbi]}'"
			break
		fi
		# Menu Item Delete from MD900
		if [[ $line =~ .+91.+83 ]]; then
			if [ $dkmd900i -gt 0 ]; then
				let "dkmd900x-=1"
				dkmd900i=$(($dkmd900x/3))
				setpos 27 28
				writetext "                    "
				setpos 27 28
				writetext "${DKMD900DIRS[$dkmd900i]}"
			fi
		fi
		if [[ $line =~ .+91.+82 ]]; then
			if [ $dkmd900i -lt $DKMD900COUNT ]; then
				let "dkmd900x+=1"
				dkmd900i=$(($dkmd900x/3))
				setpos 27 28
				writetext "                    "
				setpos 27 28
				writetext "${DKMD900DIRS[$dkmd900i]}"
			fi
		fi
		if [[ $line =~ .+91.+84 ]]; then
			res="DeleteFromMD900"
			eval $__dk="'${DKMD900DIRS[$dkmd900i]}'"
			break
		fi 
		#if [[ $line =~ .+92.+84 ]]; then
		#  res="?"
		#  break
		#fi 
		if [[ $line =~ .+93.+84 ]]; then
			res="Exit"
			break
		fi 
	#done < <(candump $CANDEV,90:7FF,91:7FF,92:7FF,93:7FF)
	done < <(candump $CANDEV,90:7FF,91:7FF,93:7FF)

	setpos 26 0
	tput -T xterm-256color el > $SCREEN
	setpos 27 0
	tput -T xterm-256color el > $SCREEN
	setpos 28 0
	tput -T xterm-256color el > $SCREEN

  eval $__res="'$res'"
}

doDeleteFromMD900 () {

  DK=$1

  DKBASEDIR=/var/lib/emi/working/wave/drumkit

  if [ -d "$DKBASEDIR/$DK" ]; then
    writetext2pos 6 2 "--> Delete drumkit '$DK' from MD900, please wait ..."
		setpos 10 2
    rm -r $DKBASEDIR/$DK > $SCREEN
    sync
    writetext2pos 10 2 ""
    writetext2pos 6 2 "--> Drumkit '$DK' deleted."
		sleep 2
  else
    writetext2pos 6 2 "Drumkit directory not found on MD900 Stick."
    writetext2pos 7 2 "$DKBASEDIR/$DK"
    sleep 5
  fi
}

doCopyToMD900 () {

  DK=$1
  
  DKBASEDIR=$(ls -d /media/*/ | head -1)

  if [ -d "$DKBASEDIR/md900-drumkits/$DK" ]; then
    writetext2pos 6 2 '<-- Copy drumkit from USB $DK directory, please wait ...'
    setpos 10 2
    rsync -r --del -L --info=progress2 $DKBASEDIR/md900-drumkits/$DK/ /var/lib/emi/working/wave/drumkit/$DK > $SCREEN
    writetext2pos 10 2 ""
    writetext2pos 6 2 '--> Write to Disk, please wait ...'
    sync
    writetext2pos 6 2 '--> Write to Disk, please wait ...ok'    
    setPermissions
  else
    writetext2pos 6 2 "Drumkit directory not found on USB Stick."
    writetext2pos 7 2 "$DKBASEDIR/md900-drumkits/$DK"
    sleep 5
  fi
}

setPermissions () {
  chown -R emios:emios /var/lib/emi/working/wave/drumkit/
  find /var/lib/emi/working/wave/drumkit/ -type d -exec chmod 770 {} +
  find /var/lib/emi/working/wave/drumkit/ -type f -exec chmod 660 {} +
}

# --- End Helpers ---

clearscreen

ABSPATH=$(readlink -f $0)
ABSDIR=$(dirname $ABSPATH)

cd $ABSDIR
zcat bg.dat > /dev/fb0

writetext2pos 6 2 'MD900 DrumKit Sample Manager'

DKBASEDIR=$(ls -d /media/*/ | head -1)
if [ -d "$DKBASEDIR/md900-drumkits" ]; then
	while true; do

		qrundkm action dk	
		
		case $action in

			CopyToMD900)
				doCopyToMD900 $dk
				action=""
				dk=""
				;;
			DeleteFromMD900)
				doDeleteFromMD900 $dk
				action=""
				dk=""
				;;
			*)
				writetext2pos 6 2 'Exit MD900 DrumKit Sample Manager ...'
				action=""
				dk=""
				sleep 1
				break
				;;
		esac
	done

	writetext2pos 6 2 'DrumKit Sample Manager job done.'
	sleep 1

else
	writetext2pos 6 2 "Drumkits directory not found on USB Stick. Exiting."
	writetext2pos 7 2 "$DKBASEDIR/md900-drumkits"		
	sleep 5
fi
